#********************************************************************************
# simplify usage of qemu monitor                                                *
#                                                                               *
#  Author: Kai Peter, ©2016-                                                    *
# Version: 0.38                                                                 *
# License: ISC                                                                  *
#                                                                               *
# Description: qemu-mon: connect to qemu monitor via qtcpclient                 *
#********************************************************************************
VERSION=`cat $0 | grep -m 1 'Version' | cut -d\   -f3`       # get version number
ME=`basename $0`
PATH=`dirname $0`:$PATH
CFF="REGFILE"

[ ! "$1" ] && echo "Usage: $ME <name> [ opts ]" && exit 1

monitor() {
  # allow one connection to the monitor only
  CONN=`netstat -an | grep $PORT | grep -m 1 ESTABL`
  [ "$CONN" ] && echo "There seems to be another connection active ..." && exit 1

  printf "\033[1mConnected! Press CTRL-C to exit qemu-monitor!\033[0m\n"
  qtcpclient localhost $PORT qtcpconn-io
  exit $?
}

getport() {
  # get port number from pidfile
  PIDFILE=`echo $VM | cut -d. -f1`.pid
  [ ! -f "$PIDFILE" ] && echo "pidfile not found!" && exit 1
  PORT=`cat $PIDFILE | grep MON | cut -d= -f2`
  [ "$PORT" ] || PORT=0
}

[ ! -f "$CFF" ] && printf "error: couldn't find config file $CFF!\n" && exit 1

PORT=0 ; TMP=""
while IFS= read VM ; do
  [ -n "$VM" ] && VM=$(echo $VM | cut -d: -f1)
  if [ -e "$VM" ] || [ -L "$VM" ] ; then
    TMP=$(echo `basename $VM` | cut -d- -f1)
    ( [ "$1" = $TMP ] || [ "$1" = $VM  ] || [ "$1" = `basename $VM`  ] ) && break
  fi
  TMP=""
done < "$CFF"

[ ! "$TMP" ] && printf "error: \033[1m$1\033[0m is not a registered VM!\n" && exit 1
getport
[ "$PORT" = 0 ] && echo "error: couldn't determine port!" && exit 1

[ "$ME" = "qemu-mon" ] && monitor

echo "error: couldn't connect to '$1'"
exit 1
